#!/usr/bin/perl -w

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2005-2005, Guardian Digital Inc.
 | $Id: gdsn_apt_method.pl,v 1.5 2005/09/21 17:15:16 pete Exp $
 |
 | Ryan W. Maple   <ryan@guardiandigital.com>
 | Peter J. O'Hara <pete@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  # use perl;
  use Digest::MD5;
  use HTTP::Date	qw(time2str str2time);
  use WebTool;
  use File::Basename;
# use strict;

  #
  # We need to use methods from the "gdsn" WebTool module so load it into
  # our namespace before we even begin.
  #
  BEGIN { WebTool::load_module('gdsn'); };

  #
  # Local prototypes.
  #
  sub acquire($$$);
  sub configure($$$);
  sub write_payload($$$$$);

  #
  # XXX
  #
# open STDERR, '/dev/null';
  $|		= 1;
  my $gdsn	= new gdsn;
  my $VERSION	= '0.1';
  my $ua	= undef; # RWMXXX
  my $message	= {};
 
  # 
  # XXX
  # 
  print "100 Capabilities\n";
  print "Version: $VERSION\n";
  print "Send-Config: true\n";
  print "Pipeline: false\n";
  print "\n";
  
  # 
  # XXX
  # 
  while (my $line = <>) {
    chomp $line;
    
    # parse first line
    unless (defined $message->{code}) {
      ($message->{code}) = $line =~ /^(\d{3}) .*/;
      err("invalid APT message syntax, aborting\n") unless defined $message->{code};
      next;
    }
    
    # end of input - act upon message
    if ($line =~ /^$/) {
      switch: for ($message->{code}) {
        acquire($message, $ua, $gdsn), last switch if /600/;
        configure($message, $ua, $gdsn), last switch if /601/;
        credentials($message, $ua, $gdsn), last switch if /602/;
        mediachange($message, $ua, $gdsn), last switch if /603/;
        unknown($message, $ua, $gdsn), last switch;
      }
      $message = {};
      next;
    }
  
    # parse input
    my ($k, $v) = split ': ', $line, 2;
    if ($k eq 'Config-Item') {
      ($k, $v) = split '=', $v, 2;
      $message->{config}{$k} = $v;
    } else {
      $message->{$k} = $v;
    }
  }
  
  exit 0;

################################################################################
sub acquire ($$$) {
  my $message	= shift;
  my $ua	= shift;
  my $gdsn	= shift;
  my $uri	= $message->{'URI'};
  my $file	= $message->{'Filename'};

  #
  # XXX
  #
  if ($uri =~ /^gdsn:\/\/([^\/]+)\/(.+)$/) {
    my $host		= $1;
    my $md5ctx		= new Digest::MD5;

    # XXX - hacks for testing
    my $fcn		= undef;
    my $distro		= undef;
    # my $comp		= undef;
    my $comp		= 'core';
    my $pkglist		= undef;
    my $arch		= undef;
   
    open (ARCH, "uname -m |") or die "Can't get determine architecture: $!";
    while (<ARCH>) {
      $arch = $_;
    }
    close ARCH;
    chomp($arch);

    # If URI ends in an rpm then the function is getpack otherwise it's payload
    if ($uri =~ /^gdsn:\/\/([^\/]+)\/(.*\.rpm)$/) {
      $fcn		= 'getpack';
    } else { 
      $fcn		= 'payload';
    }
      
    my $base_file	= basename($message->{'Filename'});
    my $package		= basename($message->{'URI'});
    
    #
    # XXX
    #
    if ($fcn eq 'payload') {		
      #
      # XXX
      #
      if ($base_file =~ m/^([^\_]+)\_GDSNROOT\_([^\_]+)\_base\_(.+)$/) {
        my $host	= $1;
        $distro		= $2;
        $base_file	= $3;
      }

      #
      # XXX 
      #
      if ($base_file =~ m/^pkglist/) { $base_file .= ".bz2"; }

    }

    #
    # XXX
    #
    elsif ($fcn eq 'getpack') {
      if ($uri =~ m/^gdsn:\/\/([^\/]+)\/GDSNROOT\/([^\/]+)\/RPMS\.([^\.]+)\/(.+)$/) {
        $distro	= $2;
        $comp	= $3;
      }
    }

    #
    # XXX
    #
    my $script	= 'gdsn_proxy_v3.cgi';
    my %args	= (
	'fcn'		=> $fcn,
	'distro'	=> $distro,
        'arch'		=> $arch,
	'component'	=> $comp,
	'base_file'	=> $base_file,
	'package'	=> $package
    );

    #
    # XXX
    #
    printf "102 Status\nURI: %s\nMessage: Connecting to %s\n\n", $uri, $host;

    my ($rv, $got)	= $gdsn->gdsn_get(\%args, $script);

    #
    # XXX
    #
    if ($rv && $rv != 0) {
      print "401 General failure\nMessage: GDSN returned error code $rv\n\n";
      return 1;
    }

    #
    # XXX
    #
    my $size	= write_payload($uri, $file, $got, $md5ctx, undef);

    #
    # XXX
    #
    printf "201 URI Done\n";
    printf "URI: %s\n", $uri;
    printf "Filename: %s\n", $file;
    printf "Size: %d\n", $size;
    printf "MD5-Hash: %s\n", $md5ctx->hexdigest;
    printf "\n";
  }
  else {
    print "401 General failure\nMessage: Invalid URI\n\n";
    return 1;
  }

  return 0;
}

################################################################################
sub configure ($$$) { }

################################################################################
sub write_payload($$$$$) {
  my $uri	= shift;
  my $file	= shift;
  my $payload	= shift;
  my $md5ctx	= shift;
  my $sha1ctx	= shift;

  #
  # XXX
  #
  open  FILE, ">$file" or die "Could not write to '$file': $!\n";

  #
  # XXX
  #
  printf "200 URI Start\n";
  printf "URI: %s\n", $uri;
  printf "Size: %d\n", (stat($file))[7];
  printf "Last-Modified: %s\n", time2str(time());
  printf "\n";

  #
  # XXX
  #
  print FILE $payload;
  close FILE;

  #
  # XXX
  #
  $md5ctx->add($payload);

  return (stat($file))[7];
}
